"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.comment = exports.update = exports.upload = void 0;
const types_1 = require("./types");
const puppeteer_extra_1 = __importDefault(require("puppeteer-extra"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const StealthPlugin = require('puppeteer-extra-plugin-stealth')();
StealthPlugin.enabledEvasions.delete('iframe.contentWindow');
StealthPlugin.enabledEvasions.delete('navigator.plugins');
puppeteer_extra_1.default.use(StealthPlugin);
const maxTitleLen = 100;
const maxDescLen = 5000;
const timeout = 60000;
const height = 900;
const width = 900;
let browser, page;
let cookiesDirPath;
let cookiesFilePath;
const invalidCharacters = ['<', '>'];
const uploadURL = 'https://www.youtube.com/upload';
const homePageURL = 'https://www.youtube.com';
const defaultMessageTransport = {
    log: console.log,
    userAction: console.log
};
/**
 * import { upload } from 'youtube-videos-uploader'
 * or
 * const { upload } = require('youtube-videos-uploader');
 */
const upload = (credentials, videos, puppeteerLaunch, messageTransport = defaultMessageTransport) => __awaiter(void 0, void 0, void 0, function* () {
    cookiesDirPath = path_1.default.join('.', 'yt-auth');
    cookiesFilePath = path_1.default.join(cookiesDirPath, `cookies-${credentials.email.split('@')[0].replace(/\./g, '_')}-${credentials.email
        .split('@')[1]
        .replace(/\./g, '_')}.json`);
    yield launchBrowser(puppeteerLaunch);
    yield loadAccount(credentials, messageTransport);
    const uploadedYTLink = [];
    for (const video of videos) {
        const link = yield uploadVideo(video, messageTransport);
        const { onSuccess } = video;
        if (typeof onSuccess === 'function') {
            onSuccess(link);
        }
        uploadedYTLink.push(link);
    }
    yield browser.close();
    return uploadedYTLink;
});
exports.upload = upload;
// `videoJSON = {}`, avoid `videoJSON = undefined` throw error.
function uploadVideo(videoJSON, messageTransport) {
    return __awaiter(this, void 0, void 0, function* () {
        const pathToFile = videoJSON.path;
        if (!pathToFile) {
            throw new Error("function `upload`'s second param `videos`'s item `video` must include `path` property.");
        }
        for (let i in invalidCharacters)
            if (videoJSON.title.includes(invalidCharacters[i]))
                throw new Error(`"${videoJSON.title}" includes a character not allowed in youtube titles (${invalidCharacters[i]})`);
        if (videoJSON.channelName) {
            yield changeChannel(videoJSON.channelName);
        }
        const title = videoJSON.title;
        const description = videoJSON.description;
        const tags = videoJSON.tags;
        // For backward compatablility playlist.name is checked first
        const playlistName = videoJSON.playlist;
        const videoLang = videoJSON.language;
        const thumb = videoJSON.thumbnail;
        const uploadAsDraft = videoJSON.uploadAsDraft;
        yield page.evaluate(() => {
            window.onbeforeunload = null;
        });
        yield page.goto(uploadURL);
        const closeBtnXPath = "//*[normalize-space(text())='Close']";
        const selectBtnXPath = "//*[normalize-space(text())='Select files']";
        const saveCloseBtnXPath = '//*[@aria-label="Save and close"]/tp-yt-iron-icon';
        const createBtnXPath = '//*[@id="create-icon"]/tp-yt-iron-icon';
        const addVideoBtnXPath = '//*[@id="text-item-0"]/ytcp-ve/div/div/yt-formatted-string';
        if ((yield page.waitForXPath(createBtnXPath).catch(() => null))) {
            const createBtn = yield page.$x(createBtnXPath);
            yield createBtn[0].click();
        }
        if ((yield page.waitForXPath(addVideoBtnXPath).catch(() => null))) {
            const addVideoBtn = yield page.$x(addVideoBtnXPath);
            yield addVideoBtn[0].click();
        }
        for (let i = 0; i < 2; i++) {
            try {
                yield page.waitForXPath(selectBtnXPath);
                yield page.waitForXPath(closeBtnXPath);
                break;
            }
            catch (error) {
                const nextText = i === 0 ? ' trying again' : ' failed again';
                messageTransport.log('Failed to find the select files button' + nextText);
                messageTransport.log(error);
                yield page.evaluate(() => {
                    window.onbeforeunload = null;
                });
                yield page.goto(uploadURL);
            }
        }
        // Remove hidden closebtn text
        const closeBtn = yield page.$x(closeBtnXPath);
        yield page.evaluate((el) => {
            el.textContent = 'oldclosse';
        }, closeBtn[0]);
        const selectBtn = yield page.$x(selectBtnXPath);
        const [fileChooser] = yield Promise.all([
            page.waitForFileChooser(),
            selectBtn[0].click() // button that triggers file selection
        ]);
        yield fileChooser.accept([pathToFile]);
        // Setup onProgress
        let progressChecker;
        let progress = { progress: 0, stage: types_1.ProgressEnum.Uploading };
        if (videoJSON.onProgress) {
            videoJSON.onProgress(progress);
            progressChecker = setInterval(() => __awaiter(this, void 0, void 0, function* () {
                let curProgress = yield page.evaluate(() => {
                    let items = document.querySelectorAll("span.progress-label.ytcp-video-upload-progress");
                    for (let i = 0; i < items.length; i++) {
                        if (items.item(i).textContent.indexOf("%") === -1)
                            continue;
                        return items.item(i).textContent;
                    }
                });
                if (progressChecker == undefined || !curProgress)
                    return;
                curProgress = curProgress.split(" ").find((txt) => txt.indexOf("%") != -1);
                let newProgress = curProgress ? parseInt(curProgress.slice(0, -1)) : 0;
                if (progress.progress == newProgress)
                    return;
                progress.progress = newProgress;
                videoJSON.onProgress(progress);
            }), 500);
        }
        const errorMessage = yield page.evaluate(() => { var _a; return (_a = document.querySelector('.error-area.style-scope.ytcp-uploads-dialog')) === null || _a === void 0 ? void 0 : _a.innerText.trim(); });
        if (errorMessage) {
            yield browser.close();
            throw new Error('Youtube returned an error : ' + errorMessage);
        }
        // Check if daily upload limit is reached
        page.waitForXPath('//*[contains(text(),"Daily upload limit reached")]', { timeout: 0 })
            .then(() => { browser.close().then(() => { throw new Error('Daily upload limit reached'); }); })
            .catch(() => { });
        // Check if processing abandoned
        page.waitForXPath('//*[contains(text(),"Processing abandoned")]', { timeout: 0 })
            .then(() => { browser.close().then(() => { throw new Error('Processing abandoned'); }); })
            .catch(() => { });
        // Wait for upload to complete
        yield page.waitForXPath('//*[contains(text(),"Upload complete")]', { timeout: 0 }).then(() => sleep(2000).then(() => 'uploadComplete'));
        // Wait for upload to go away and processing to start, skip the wait if the user doesn't want it.
        if (!videoJSON.skipProcessingWait) {
            yield page.waitForXPath('//*[contains(text(),"Upload complete")]', { hidden: true, timeout: 0 });
        }
        else {
            yield sleep(5000);
        }
        if (videoJSON.onProgress) {
            progress = { progress: 0, stage: types_1.ProgressEnum.Processing };
            videoJSON.onProgress(progress);
        }
        if (videoJSON.onProgress) {
            clearInterval(progressChecker);
            progressChecker = undefined;
            progress = { progress: 100, stage: types_1.ProgressEnum.Done };
            videoJSON.onProgress(progress);
        }
        // Wait until title & description box pops up
        if (thumb) {
            let thumbnailChooserXpath = xpathTextSelector("upload thumbnail");
            yield page.waitForXPath(thumbnailChooserXpath);
            const thumbBtn = yield page.$x(thumbnailChooserXpath);
            const [thumbChooser] = yield Promise.all([
                page.waitForFileChooser(),
                thumbBtn[0].click() // button that triggers file selection
            ]);
            yield thumbChooser.accept([thumb]);
        }
        yield page.waitForFunction('document.querySelectorAll(\'[id="textbox"]\').length > 1');
        const textBoxes = yield page.$x('//*[@id="textbox"]');
        yield page.bringToFront();
        // Add the title value
        yield textBoxes[0].focus();
        yield page.waitForTimeout(1000);
        yield textBoxes[0].type(title.substring(0, maxTitleLen));
        // Add the Description content
        yield textBoxes[1].type(description.substring(0, maxDescLen));
        const childOption = yield page.$x('//*[contains(text(),"No, it\'s")]');
        yield childOption[0].click();
        const moreOption = yield page.$x("//*[normalize-space(text())='Show more']");
        yield moreOption[0].click();
        const playlist = yield page.$x("//*[normalize-space(text())='Select']");
        let createplaylistdone;
        if (playlistName) {
            // Selecting playlist
            for (let i = 0; i < 2; i++) {
                try {
                    yield page.evaluate((el) => el.click(), playlist[0]);
                    // Type the playlist name to filter out
                    yield page.waitForSelector('#search-input');
                    yield page.focus(`#search-input`);
                    yield page.type(`#search-input`, playlistName);
                    const escapedPlaylistName = escapeQuotesForXPath(playlistName);
                    const playlistToSelectXPath = "//*[normalize-space(text())=" + escapedPlaylistName + "]";
                    yield page.waitForXPath(playlistToSelectXPath, { timeout: 10000 });
                    const playlistNameSelector = yield page.$x(playlistToSelectXPath);
                    yield page.evaluate((el) => el.click(), playlistNameSelector[0]);
                    createplaylistdone = yield page.$x("//*[normalize-space(text())='Done']");
                    yield page.evaluate((el) => el.click(), createplaylistdone[0]);
                    break;
                }
                catch (error) {
                    // Creating new playlist
                    // click on playlist dropdown
                    yield page.evaluate((el) => el.click(), playlist[0]);
                    // click New playlist button
                    const newPlaylistXPath = "//*[normalize-space(text())='New playlist'] | //*[normalize-space(text())='Create playlist']";
                    yield page.waitForXPath(newPlaylistXPath);
                    const createplaylist = yield page.$x(newPlaylistXPath);
                    yield page.evaluate((el) => el.click(), createplaylist[0]);
                    // Enter new playlist name
                    yield page.keyboard.type(' ' + playlistName.substring(0, 148));
                    // click create & then done button
                    const createplaylistbtn = yield page.$x("//*[normalize-space(text())='Create']");
                    yield page.evaluate((el) => el.click(), createplaylistbtn[1]);
                    createplaylistdone = yield page.$x("//*[normalize-space(text())='Done']");
                    yield page.evaluate((el) => el.click(), createplaylistdone[0]);
                }
            }
        }
        // Add tags
        if (tags) {
            yield page.focus(`[aria-label="Tags"]`);
            yield page.type(`[aria-label="Tags"]`, tags.join(', ').substring(0, 495) + ', ');
        }
        // Selecting video language
        if (videoLang) {
            const langHandler = yield page.$x("//*[normalize-space(text())='Video language']");
            yield page.evaluate((el) => el.click(), langHandler[0]);
            // translate(text(),'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')
            const langName = yield page.$x('//*[normalize-space(translate(text(),"ABCDEFGHIJKLMNOPQRSTUVWXYZ","abcdefghijklmnopqrstuvwxyz"))=\'' +
                videoLang.toLowerCase() +
                "']");
            yield page.evaluate((el) => el.click(), langName[langName.length - 1]);
        }
        // click next button
        const nextBtnXPath = "//*[normalize-space(text())='Next']/parent::*[not(@disabled)]";
        yield page.waitForXPath(nextBtnXPath);
        let next = yield page.$x(nextBtnXPath);
        yield next[0].click();
        // await sleep(2000)
        yield page.waitForXPath(nextBtnXPath);
        // click next button
        next = yield page.$x(nextBtnXPath);
        yield next[0].click();
        yield page.waitForXPath(nextBtnXPath);
        // click next button
        next = yield page.$x(nextBtnXPath);
        yield next[0].click();
        //  const publicXPath = `//*[normalize-space(text())='Public']`
        //  await page.waitForXPath(publicXPath)
        //  const publicOption = await page.$x(publicXPath)
        //  await publicOption[0].click()
        // Get publish button
        const publishXPath = "//*[normalize-space(text())='Publish']/parent::*[not(@disabled)] | //*[normalize-space(text())='Save']/parent::*[not(@disabled)]";
        yield page.waitForXPath(publishXPath);
        // save youtube upload link
        const videoBaseLink = 'https://youtu.be';
        const shortVideoBaseLink = 'https://youtube.com/shorts';
        const uploadLinkSelector = `[href^="${videoBaseLink}"], [href^="${shortVideoBaseLink}"]`;
        yield page.waitForSelector(uploadLinkSelector);
        const uploadedLinkHandle = yield page.$(uploadLinkSelector);
        let uploadedLink;
        do {
            yield page.waitForTimeout(500);
            uploadedLink = yield page.evaluate((e) => e.getAttribute('href'), uploadedLinkHandle);
        } while (uploadedLink === videoBaseLink || uploadedLink === shortVideoBaseLink);
        const closeDialogXPath = uploadAsDraft ? saveCloseBtnXPath : publishXPath;
        let closeDialog;
        for (let i = 0; i < 10; i++) {
            try {
                closeDialog = yield page.$x(closeDialogXPath);
                yield closeDialog[0].click();
                break;
            }
            catch (error) {
                yield page.waitForTimeout(5000);
            }
        }
        // await page.waitForXPath('//*[contains(text(),"Finished processing")]', { timeout: 0})
        // no closeBtn will show up if keeps video as draft
        if (uploadAsDraft)
            return uploadedLink;
        // Wait for closebtn to show up
        try {
            yield page.waitForXPath(closeBtnXPath);
        }
        catch (e) {
            yield browser.close();
            throw new Error('Please make sure you set up your default video visibility correctly, you might have forgotten. More infos : https://github.com/fawazahmed0/youtube-uploader#youtube-setup');
        }
        return uploadedLink;
    });
}
const update = (credentials, videos, puppeteerLaunch, messageTransport = defaultMessageTransport) => __awaiter(void 0, void 0, void 0, function* () {
    cookiesDirPath = path_1.default.join('.', 'yt-auth');
    cookiesFilePath = path_1.default.join(cookiesDirPath, `cookies-${credentials.email.split('@')[0].replace(/\./g, '_')}-${credentials.email
        .split('@')[1]
        .replace(/\./g, '_')}.json`);
    yield launchBrowser(puppeteerLaunch);
    if (!fs_extra_1.default.existsSync(cookiesFilePath))
        yield loadAccount(credentials, messageTransport);
    const updatedYTLink = [];
    for (const video of videos) {
        messageTransport.log(video);
        const link = yield updateVideoInfo(video, messageTransport);
        const { onSuccess } = video;
        if (typeof onSuccess === 'function') {
            onSuccess(link);
        }
        updatedYTLink.push(link);
    }
    yield browser.close();
    return updatedYTLink;
});
exports.update = update;
const comment = (credentials, comments, puppeteerLaunch, messageTransport = defaultMessageTransport) => __awaiter(void 0, void 0, void 0, function* () {
    cookiesDirPath = path_1.default.join('.', 'yt-auth');
    cookiesFilePath = path_1.default.join(cookiesDirPath, `cookies-${credentials.email.split('@')[0].replace(/\./g, '_')}-${credentials.email
        .split('@')[1]
        .replace(/\./g, '_')}.json`);
    yield launchBrowser(puppeteerLaunch);
    if (!fs_extra_1.default.existsSync(cookiesFilePath))
        yield loadAccount(credentials, messageTransport);
    const commentsS = [];
    for (const comment of comments) {
        let result;
        messageTransport.log(comment);
        if (comment.live)
            result = yield publishLiveComment(comment, messageTransport);
        else
            result = yield publishComment(comment);
        const { onSuccess } = comment;
        if (typeof onSuccess === 'function') {
            onSuccess(result);
        }
        commentsS.push(result);
    }
    yield browser.close();
    return commentsS;
});
exports.comment = comment;
const publishComment = (comment) => __awaiter(void 0, void 0, void 0, function* () {
    const videoUrl = comment.link;
    if (!videoUrl) {
        throw new Error('The link of the video is a required parameter');
    }
    try {
        const cmt = comment.comment;
        yield page.goto(videoUrl);
        yield sleep(2000);
        yield scrollTillVeiw(page, `#placeholder-area`);
        yield page.focus(`#placeholder-area`);
        const commentBox = yield page.$x('//*[@id="placeholder-area"]');
        yield commentBox[0].focus();
        yield commentBox[0].click();
        yield commentBox[0].type(cmt.substring(0, 10000));
        yield page.click('#submit-button');
        if (comment.pin) {
            yield pinComment(cmt);
        }
        return { err: false, data: 'sucess' };
    }
    catch (err) {
        return { err: true, data: err };
    }
});
const pinComment = (comment) => __awaiter(void 0, void 0, void 0, function* () {
    const actionMenuXPath = `//div[@id="primary"]//div[@id="body" and contains(., "${comment}")]//button[@id="button" and @aria-label="Action menu"]`;
    const pinButtonXPath = '//yt-formatted-string[contains(@class, "ytd-menu-navigation-item-renderer") and contains(., "Pin")]';
    const confirmPinButtonXPath = '//div[@id="main" and contains(., "Pin this comment?")]//a[contains(., "Pin")]';
    // const confirmPinButtonXPath = '//yt-formatted-string[contains(@class, "yt-button-renderer") and contains(., "Pin")]';
    yield page.waitForXPath(actionMenuXPath);
    const [actionMenu] = yield page.$x(actionMenuXPath);
    yield actionMenu.evaluate(x => x.click());
    yield page.waitForXPath(pinButtonXPath);
    const [pinButton] = yield page.$x(pinButtonXPath);
    yield pinButton.click();
    yield page.waitForXPath(confirmPinButtonXPath);
    const [confirmPinButton] = yield page.$x(confirmPinButtonXPath);
    yield confirmPinButton.click();
});
const publishLiveComment = (comment, messageTransport) => __awaiter(void 0, void 0, void 0, function* () {
    const videoUrl = comment.link;
    const cmt = comment.comment;
    if (!videoUrl) {
        throw new Error('The link of the video is a required parameter');
    }
    yield page.goto(videoUrl);
    yield sleep(3000);
    yield scrollTillVeiw(page, `#label`);
    try {
        yield page.focus(`#label`);
    }
    catch (err) {
        messageTransport.log(err);
        throw new Error('Video may not be Live');
    }
    for (let i = 0; i < 6; i++) {
        yield autoScroll(page);
    }
    try {
        yield page.focus('#input');
        yield page.mouse.click(450, 480);
        yield page.keyboard.type(cmt.substring(0, 200));
        yield sleep(200);
        yield page.mouse.click(841, 495);
        return { err: false, data: 'sucess' };
    }
    catch (err) {
        return { err: true, data: err };
    }
});
const updateVideoInfo = (videoJSON, messageTransport) => __awaiter(void 0, void 0, void 0, function* () {
    const videoUrl = videoJSON.link;
    if (!videoUrl) {
        throw new Error('The link of the  video is a required parameter');
    }
    if (videoJSON.channelName) {
        yield changeChannel(videoJSON.channelName);
    }
    const title = videoJSON.title;
    const description = videoJSON.description;
    const tags = videoJSON.tags;
    const Rtags = videoJSON.replaceTags;
    const playlistName = videoJSON.playlist;
    const videoLang = videoJSON.language;
    const thumb = videoJSON.thumbnail;
    const publish = videoJSON.publishType;
    yield page.goto(videoUrl);
    const editXpath = '//*[@id="subscribe-button"]/ytd-button-renderer';
    try {
        yield page.waitForXPath(editXpath, { timeout: 7000 });
    }
    catch (err) {
        throw new Error('The video provided may not be yours');
    }
    let edit = yield page.$x(editXpath);
    yield edit[0].click();
    const titleE = '//*[@id="textbox"]';
    yield page.waitForXPath(titleE, { timeout: 70000 });
    yield page.waitForFunction('document.querySelectorAll(\'[id="textbox"]\').length > 1');
    const textBoxes = yield page.$x('//*[@id="textbox"]');
    yield page.bringToFront();
    // Edit the title value (if)
    yield textBoxes[0].focus();
    yield page.waitForTimeout(1000);
    yield sleep(1000);
    if (title) {
        yield page.keyboard.down('Control');
        yield page.keyboard.press('A');
        yield page.keyboard.up('Control');
        yield page.keyboard.press('Backspace');
        yield textBoxes[0].type(title.substring(0, maxTitleLen));
    }
    // Edit the Description content (if)
    if (description) {
        yield textBoxes[1].focus();
        yield page.keyboard.down('Control');
        yield page.keyboard.press('A');
        yield page.keyboard.up('Control');
        yield page.keyboard.press('Backspace');
        yield textBoxes[1].type(description.substring(0, maxDescLen));
    }
    if (thumb) {
        const [thumbChooser] = yield Promise.all([
            page.waitForFileChooser({ timeout: 500 }).catch(() => __awaiter(void 0, void 0, void 0, function* () {
                messageTransport.log('replacing previous thumbanail');
                yield page.click('#still-1 > button');
                yield page.waitForSelector('#save > div');
                yield page.click(`#save > div`);
                yield page.waitForXPath("//*[normalize-space(text())='Save']/parent::*[@disabled]");
                yield sleep(500);
                return yield page.waitForFileChooser();
            })),
            yield page.waitForSelector(`[class="remove-default-style style-scope ytcp-thumbnails-compact-editor-uploader"]`),
            yield page.click(`[class="remove-default-style style-scope ytcp-thumbnails-compact-editor-uploader"]`)
        ]);
        yield thumbChooser.accept([thumb]);
    }
    // await sleep( 10000000)
    const playlist = yield page.$x(`//*[@id="basics"]/div[4]/div[3]/div[1]/ytcp-video-metadata-playlists/ytcp-text-dropdown-trigger/ytcp-dropdown-trigger/div/div[3]`);
    let createplaylistdone;
    if (playlistName) {
        for (let i = 0; i < 2; i++) {
            try {
                yield page.evaluate((el) => el.click(), playlist[0]);
                yield page.waitForSelector('#search-input');
                yield page.focus(`#search-input`);
                yield page.type(`#search-input`, playlistName);
                const escapedPlaylistName = escapeQuotesForXPath(playlistName);
                const playlistToSelectXPath = "//*[normalize-space(text())=" + escapedPlaylistName + "]";
                yield page.waitForXPath(playlistToSelectXPath, { timeout: 10000 });
                const playlistNameSelector = yield page.$x(playlistToSelectXPath);
                yield page.evaluate((el) => el.click(), playlistNameSelector[0]);
                createplaylistdone = yield page.$x("//*[normalize-space(text())='Done']");
                yield page.evaluate((el) => el.click(), createplaylistdone[0]);
                break;
            }
            catch (error) {
                yield page.evaluate((el) => el.click(), playlist[0]);
                const newPlaylistXPath = "//*[normalize-space(text())='New playlist'] | //*[normalize-space(text())='Create playlist']";
                yield page.waitForXPath(newPlaylistXPath);
                const createplaylist = yield page.$x(newPlaylistXPath);
                yield page.evaluate((el) => el.click(), createplaylist[0]);
                yield page.keyboard.type(' ' + playlistName.substring(0, 148));
                const createplaylistbtn = yield page.$x("//*[normalize-space(text())='Create']");
                yield page.evaluate((el) => el.click(), createplaylistbtn[1]);
                createplaylistdone = yield page.$x("//*[normalize-space(text())='Done']");
                yield page.evaluate((el) => el.click(), createplaylistdone[0]);
            }
        }
    }
    const moreOption = yield page.$x("//*[normalize-space(text())='Show more']");
    yield moreOption[0].click();
    if (tags) {
        yield page.focus(`[aria-label="Tags"]`);
        yield page.type(`[aria-label="Tags"]`, tags.join(', ').substring(0, 495) + ', ');
    }
    if (Rtags) {
        yield page.click('//*[@id="clear-button"]/tp-yt-iron-icon');
        yield page.focus(`[aria-label="Tags"]`);
        yield page.type(`[aria-label="Tags"]`, Rtags.join(', ').substring(0, 495) + ', ');
    }
    if (videoLang) {
        const langHandler = yield page.$x("//*[normalize-space(text())='Video language']");
        yield page.evaluate((el) => el.click(), langHandler[0]);
        const langName = yield page.$x('//*[normalize-space(translate(text(),"ABCDEFGHIJKLMNOPQRSTUVWXYZ","abcdefghijklmnopqrstuvwxyz"))=\'' +
            videoLang.toLowerCase() +
            "']");
        yield page.evaluate((el) => el.click(), langName[langName.length - 1]);
    }
    yield page.focus(`#content`);
    if (publish) {
        yield page.click(`#content`);
        // await page.click(`#onRadio`);
        const publishBtn = yield page.$x('//*[@id="first-container"]');
        yield sleep(2000);
        // publishBtn[0].click()
        try {
            switch (publish) {
                case 'private':
                    yield page
                        .click(`#privacy-radios > tp-yt-paper-radio-button:nth-child(2)`)
                        .catch((err) => __awaiter(void 0, void 0, void 0, function* () {
                        return yield page.click(`#privacy-radios > tp-yt-paper-radio-button.style-scope.ytcp-video-visibility-select.iron-selected`);
                    }));
                    break;
                case 'unlisted':
                    yield page
                        .click(`#privacy-radios > tp-yt-paper-radio-button.style-scope.ytcp-video-visibility-select.iron-selected`)
                        .catch((err) => __awaiter(void 0, void 0, void 0, function* () { return yield page.click(`#privacy-radios > tp-yt-paper-radio-button:nth-child(11)`); }));
                    break;
                case 'public':
                    yield page
                        .click(`#privacy-radios > tp-yt-paper-radio-button:nth-child(15)`)
                        .catch((err) => __awaiter(void 0, void 0, void 0, function* () { return yield page.click(`#privacy-radios > tp-yt-paper-radio-button:nth-child(16)`); }));
                    break;
                case 'public&premiere':
                    yield page.click(`#privacy-radios > tp-yt-paper-radio-button:nth-child(15)`);
                    yield page.click(`#enable-premiere-checkbox`);
                    break;
            }
        }
        catch (err) {
            messageTransport.log('already selected');
            yield page.keyboard.press('Escape');
        }
        yield page.click(`#save-button`);
        yield sleep(1200);
    }
    try {
        yield page.focus(`#content`);
        yield page.focus(`#save > div`);
        yield page.waitForSelector('#save > div');
        yield page.click(`#save > div`);
        yield page.waitForXPath("//*[normalize-space(text())='Save']/parent::*[@disabled]");
    }
    catch (err) {
        messageTransport.log(err);
        throw new Error('Probably nothing was changed ...');
    }
    //#overflow-menu-button
    return messageTransport.log('successfully edited');
});
function loadAccount(credentials, messageTransport) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            if (!fs_extra_1.default.existsSync(cookiesFilePath))
                yield login(page, credentials, messageTransport);
        }
        catch (error) {
            if (error.message === 'Recapcha found') {
                if (browser) {
                    yield browser.close();
                }
                throw error;
            }
            // Login failed trying again to login
            try {
                yield login(page, credentials, messageTransport);
            }
            catch (error) {
                if (browser) {
                    yield browser.close();
                }
                throw error;
            }
        }
        try {
            yield changeHomePageLangIfNeeded(page);
        }
        catch (error) {
            messageTransport.log(error);
            yield login(page, credentials, messageTransport);
        }
    });
}
function changeLoginPageLangIfNeeded(localPage) {
    return __awaiter(this, void 0, void 0, function* () {
        const selectedLangSelector = '[aria-selected="true"]';
        try {
            yield localPage.waitForSelector(selectedLangSelector);
        }
        catch (e) {
            throw new Error('Failed to find selected language : ' + e.name);
        }
        const selectedLang = yield localPage.evaluate((selectedLangSelector) => document.querySelector(selectedLangSelector).innerText, selectedLangSelector);
        if (!selectedLang) {
            throw new Error('Failed to find selected language : Empty text');
        }
        if (selectedLang.includes('English')) {
            return;
        }
        yield localPage.click(selectedLangSelector);
        yield localPage.waitForTimeout(1000);
        const englishLangItemSelector = '[role="presentation"]:not([aria-hidden="true"])>[data-value="en-GB"]';
        try {
            yield localPage.waitForSelector(englishLangItemSelector);
        }
        catch (e) {
            throw new Error('Failed to find english language item : ' + e.name);
        }
        yield localPage.click(englishLangItemSelector);
        yield localPage.waitForTimeout(1000);
    });
}
function changeHomePageLangIfNeeded(localPage) {
    return __awaiter(this, void 0, void 0, function* () {
        yield localPage.goto(homePageURL);
        const avatarButtonSelector = 'button#avatar-btn';
        try {
            yield localPage.waitForSelector(avatarButtonSelector);
        }
        catch (e) {
            throw new Error('Avatar/Profile picture button not found : ' + e.name);
        }
        yield localPage.click(avatarButtonSelector);
        const langMenuItemSelector = '#sections>yt-multi-page-menu-section-renderer:nth-child(3)>#items>ytd-compact-link-renderer>a';
        try {
            yield localPage.waitForSelector(langMenuItemSelector);
        }
        catch (e) {
            throw new Error('Language menu item selector/button(">") not found : ' + e.name);
        }
        const selectedLang = yield localPage.evaluate((langMenuItemSelector) => document.querySelector(langMenuItemSelector).innerText, langMenuItemSelector);
        if (!selectedLang) {
            throw new Error('Failed to find selected language : Empty text');
        }
        if (selectedLang.includes('English')) {
            yield localPage.goto(uploadURL);
            return;
        }
        yield localPage.click(langMenuItemSelector);
        const englishItemXPath = "//*[normalize-space(text())='English (UK)']";
        try {
            yield localPage.waitForXPath(englishItemXPath);
        }
        catch (e) {
            throw new Error('English(UK) item selector not found : ' + e.name);
        }
        yield localPage.waitForTimeout(3000);
        yield localPage.evaluate((englishItemXPath) => {
            let element = document === null || document === void 0 ? void 0 : document.evaluate(englishItemXPath, document, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
            element.click();
        }, englishItemXPath);
        //Recursive language change, if YouTube, for some reason, did not change the language the first time, although the English (UK) button was pressed, the exit from the recursion occurs when the selectedLang selector is tested for the set language
        yield changeHomePageLangIfNeeded(localPage);
    });
}
function launchBrowser(puppeteerLaunch) {
    return __awaiter(this, void 0, void 0, function* () {
        const previousSession = fs_extra_1.default.existsSync(cookiesFilePath);
        browser = yield puppeteer_extra_1.default.launch(puppeteerLaunch);
        page = yield browser.newPage();
        yield page.setDefaultTimeout(timeout);
        if (previousSession) {
            // If file exist load the cookies
            const cookiesString = fs_extra_1.default.readFileSync(cookiesFilePath, { encoding: 'utf-8' });
            const parsedCookies = JSON.parse(cookiesString);
            if (parsedCookies.length !== 0) {
                for (let cookie of parsedCookies) {
                    yield page.setCookie(cookie);
                }
            }
        }
        yield page.setViewport({ width: width, height: height });
        yield page.setBypassCSP(true);
    });
}
function login(localPage, credentials, messageTransport) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        yield localPage.goto(uploadURL);
        yield changeLoginPageLangIfNeeded(localPage);
        const emailInputSelector = 'input[type="email"]';
        yield localPage.waitForSelector(emailInputSelector);
        yield localPage.type(emailInputSelector, credentials.email, { delay: 50 });
        yield localPage.keyboard.press('Enter');
        // check if 2fa code was sent to phone
        yield localPage.waitForNavigation();
        yield localPage.waitForTimeout(1000);
        const googleAppAuthSelector = 'samp';
        const isOnGoogleAppAuthPage = yield localPage.evaluate((authCodeSelector) => document.querySelector(authCodeSelector) !== null, googleAppAuthSelector);
        if (isOnGoogleAppAuthPage) {
            const codeElement = yield localPage.$('samp');
            const code = (_a = (yield (codeElement === null || codeElement === void 0 ? void 0 : codeElement.getProperty('textContent')))) === null || _a === void 0 ? void 0 : _a.toString().replace('JSHandle:', '');
            code && messageTransport.userAction('Press ' + code + ' on your phone to login');
        }
        // password isnt required in the case that a code was sent via google auth
        else {
            const passwordInputSelector = 'input[type="password"]:not([aria-hidden="true"])';
            yield localPage.waitForSelector(passwordInputSelector);
            yield localPage.waitForTimeout(3000);
            yield localPage.type(passwordInputSelector, credentials.pass, { delay: 50 });
            yield localPage.keyboard.press('Enter');
        }
        try {
            yield localPage.waitForNavigation();
            yield localPage.waitForTimeout(1000);
            // check if sms code was sent
            const smsAuthSelector = '#idvPin';
            const isOnSmsAuthPage = yield localPage.evaluate((smsAuthSelector) => document.querySelector(smsAuthSelector) !== null, smsAuthSelector);
            if (isOnSmsAuthPage) {
                try {
                    if (!messageTransport.onSmsVerificationCodeSent)
                        throw new Error('onSmsVerificationCodeSent not implemented');
                    let code = yield messageTransport.onSmsVerificationCodeSent();
                    if (!code)
                        throw new Error('Invalid SMS Code');
                    yield localPage.type(smsAuthSelector, code.trim());
                    yield localPage.keyboard.press('Enter');
                }
                catch (error) {
                    yield browser.close();
                    throw error;
                }
            }
        }
        catch (error) {
            const recaptchaInputSelector = 'input[aria-label="Type the text you hear or see"]';
            const isOnRecaptchaPage = yield localPage.evaluate((recaptchaInputSelector) => document.querySelector(recaptchaInputSelector) !== null, recaptchaInputSelector);
            if (isOnRecaptchaPage) {
                throw new Error('Recaptcha found');
            }
            throw new Error(error);
        }
        //create channel if not already created.
        try {
            yield localPage.click('#create-channel-button');
            yield localPage.waitForTimeout(3000);
        }
        catch (error) {
            messageTransport.log('Channel already exists or there was an error creating the channel.');
        }
        try {
            const uploadPopupSelector = 'ytcp-uploads-dialog';
            yield localPage.waitForSelector(uploadPopupSelector, { timeout: 70000 });
        }
        catch (error) {
            if (credentials.recoveryemail)
                yield securityBypass(localPage, credentials.recoveryemail, messageTransport);
        }
        const cookiesObject = yield localPage.cookies();
        yield fs_extra_1.default.mkdirSync(cookiesDirPath, { recursive: true });
        // Write cookies to temp file to be used in other profile pages
        yield fs_extra_1.default.writeFile(cookiesFilePath, JSON.stringify(cookiesObject), function (err) {
            if (err) {
                messageTransport.log('The file could not be written. ' + err.message);
            }
            messageTransport.log('Session has been successfully saved');
        });
    });
}
// Login bypass with recovery email
function securityBypass(localPage, recoveryemail, messageTransport) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const confirmRecoveryXPath = "//*[normalize-space(text())='Confirm your recovery email']";
            yield localPage.waitForXPath(confirmRecoveryXPath);
            const confirmRecoveryBtn = yield localPage.$x(confirmRecoveryXPath);
            yield localPage.evaluate((el) => el.click(), confirmRecoveryBtn[0]);
        }
        catch (error) {
            messageTransport.log(error);
        }
        yield localPage.waitForNavigation({
            waitUntil: 'networkidle0'
        });
        const enterRecoveryXPath = "//*[normalize-space(text())='Enter recovery email address']";
        yield localPage.waitForXPath(enterRecoveryXPath);
        yield localPage.waitForTimeout(5000);
        yield localPage.focus('input[type="email"]');
        yield localPage.waitForTimeout(3000);
        yield localPage.type('input[type="email"]', recoveryemail, { delay: 100 });
        yield localPage.keyboard.press('Enter');
        yield localPage.waitForNavigation({
            waitUntil: 'networkidle0'
        });
        const uploadPopupSelector = 'ytcp-uploads-dialog';
        yield localPage.waitForSelector(uploadPopupSelector, { timeout: 60000 });
    });
}
function sleep(ms) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((sendMessage) => setTimeout(sendMessage, ms));
    });
}
function autoScroll(page) {
    return __awaiter(this, void 0, void 0, function* () {
        yield page.evaluate(`(async () => {
        await new Promise((resolve, reject) => {
            var totalHeight = 0;
            var distance = 100;
            var timer = setInterval(() => {
                var scrollHeight = document.body.scrollHeight;
                window.scrollBy(0, distance);
                totalHeight += distance;

                if(totalHeight >= scrollHeight){
                    clearInterval(timer);
                    resolve(0);
                }
            }, 100);
        });
    })()`);
    });
}
function scrollTillVeiw(page, element) {
    return __awaiter(this, void 0, void 0, function* () {
        let sc = true;
        while (sc) {
            try {
                yield page.focus(element);
                sc = false;
            }
            catch (err) {
                yield autoScroll(page);
                sc = true;
            }
        }
        return;
    });
}
function changeChannel(channelName) {
    return __awaiter(this, void 0, void 0, function* () {
        yield page.goto("https://www.youtube.com/channel_switcher");
        const channelNameXPath = `//*[normalize-space(text())='${channelName}']`;
        const element = yield page.waitForXPath(channelNameXPath);
        yield element.click();
        yield page.waitForNavigation({
            waitUntil: "networkidle0"
        });
    });
}
function escapeQuotesForXPath(str) {
    // If the value contains only single or double quotes, construct
    // an XPath literal
    if (!str.includes('"')) {
        return '"' + str + '"';
    }
    if (!str.includes("'")) {
        return "'" + str + "'";
    }
    // If the value contains both single and double quotes, construct an 
    // expression that concatenates all non-double-quote substrings with
    // the quotes, e.g.:
    //
    //    concat("foo",'"',"bar")
    const parts = [];
    // First, put a '"' after each component in the string.
    for (const part of str.split('"')) {
        if (part.length > 0) {
            parts.push('"' + part + '"');
        }
        parts.push("'\"'");
    }
    // Then remove the extra '"' after the last component.
    parts.pop();
    // Finally, put it together into a concat() function call.
    return "concat(" + parts.join(",") + ")";
}
function xpathTextSelector(text, caseSensitive, nthElement) {
    let xpathSelector = '';
    if (caseSensitive)
        xpathSelector = `//*[contains(normalize-space(text()),"${text}")]`;
    else {
        let uniqueText = [...new Set(text.split(''))].join('');
        xpathSelector = `//*[contains(translate(normalize-space(text()),'${uniqueText.toUpperCase()}','${uniqueText.toLowerCase()}'),"${text.toLowerCase().replace(/\s\s+/g, " ")}")]`;
    }
    if (nthElement)
        xpathSelector = `(${xpathSelector})[${nthElement + 1}]`;
    return xpathSelector;
}
//# sourceMappingURL=upload.js.map